import { EventEmitter, Directive, ElementRef, Renderer2, Output, Input, HostListener, HostBinding, Pipe, InjectionToken, Injectable, Inject, Optional, NgModule } from '@angular/core';
import { BehaviorSubject, Subject, of } from 'rxjs';
import { __awaiter } from 'tslib';
import { HttpRequest, HttpEventType, HttpClient } from '@angular/common/http';
import { takeUntil, map, catchError } from 'rxjs/operators';
import { DatePipe } from '@angular/common';

// @dynamic
class Utils {
    /**
     * Creates a unique id for submit form.
     *
     * @returns string
     *
     * @memberOf utils
     */
    static uniqueID() {
        return 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, (c) => {
            // tslint:disable-next-line:no-bitwise
            const r = (Math.random() * 16) | 0, v = c === 'x' ? r : (r & 0x3) | 0x8;
            return v.toString(16);
        });
    }
    /**
     * Validate input and returns true if input is an element
     *
     * @param input any
     * @returns True if `value` is a DOM element (or wrapped jQuery element).
     *
     * @memberOf FileManager
     */
    static isElement(input) {
        return !!(input &&
            (input.nodeName || // we are a direct element
                (input.prop && input.attr && input.find))); // We have an on and find methode part of jQuery API
    }
    /**
     * Validate input and returns true if input is a string
     *
     * @param input any
     * @returns boolean
     *
     * @memberOf Utils
     */
    static isString(input) {
        return !!(typeof input === 'string');
    }
    static extendValue(...args) {
        let _value = args[0];
        for (let i = 1; i < args.length; i++) {
            if (typeof args[i] !== 'undefined') {
                _value = args[i];
            }
        }
        return _value;
    }
}
Utils._uniqueNumber = 1;
Utils.nextUID = () => (Utils._uniqueNumber++).toString();

class FileObject {
    constructor(fileOrInput) {
        const isInput = Utils.isElement(fileOrInput);
        if (isInput) {
            this._createFromFakePath(fileOrInput.value);
        }
        else {
            this._createFromObject(fileOrInput);
        }
    }
    _createFromFakePath(path) {
        this.lastModifiedDate = null;
        this.size = null;
        this.type = 'like/' + path.slice(path.lastIndexOf('.') + 1).toLowerCase();
        this.name = path.split(/(\\|\/)/g).pop();
    }
    _createFromObject(object) {
        this.lastModifiedDate = new Date(object.lastModified || object.lastModifiedDate.getTime());
        this.size = object.size;
        this.type = object.type;
        this.name = object.name;
    }
}

const FileManagerOptionsDefault = {};
const speedObject = {
    total: 0,
    loaded: 0,
    percent: 0,
    speed: 0,
    speedToText: '0 bytes'
};
class FileManager {
    /**
     * Creates an instance of FileManager.
     *
     * @memberOf FileManager
     */
    constructor(file, options, uploader) {
        this._progress$ = new BehaviorSubject(0);
        this._speed$ = new BehaviorSubject(speedObject);
        this.options = Object.assign({}, FileManagerOptionsDefault, options);
        this._speedDefault = {};
        this._id = Utils.uniqueID();
        this._isUploading = false;
        this._isUploaded = false;
        this._isSuccess = false;
        this._isCancel = false;
        this._isError = false;
        const _isInput = Utils.isElement(file);
        const _file = _isInput ? new FileObject(file.files[0]) : new FileObject(file);
        this._file = _file;
        this._fileElement = _isInput ? file.files[0] : file;
        this._fileActive = false;
        if (typeof uploader !== 'undefined') {
            this.bindUploader(uploader);
        }
    }
    set protocol(_protocol) {
        this._protocol = _protocol;
    }
    get id() {
        return this._id;
    }
    get progressPercent$() {
        return this._progress$.asObservable();
    }
    get progress$() {
        return this._speed$.asObservable();
    }
    get progress() {
        return this._progress$.getValue();
    }
    get element() {
        return this._fileElement;
    }
    get object() {
        return this._file;
    }
    get name() {
        return this._file.name;
    }
    get type() {
        return this._file.type;
    }
    get date() {
        return this._file.lastModifiedDate;
    }
    get size() {
        return this._file.size;
    }
    get inQueue() {
        return this._fileActive;
    }
    get response() {
        return this._response;
    }
    get success() {
        return this._isSuccess;
    }
    get error() {
        return this._isError;
    }
    /**
     * Bind uploader to FileManager
     *
     * @memberOf FileManager
     */
    bindUploader(uploader) {
        this._uploader = uploader;
        const check = this._uploader.addFile(this);
        this._setFileActive(check);
    }
    /**
     * Bind options to FileManager
     *
     * @memberOf FileManager
     */
    bindOptions(options) {
        const _formData = Object.assign(Object.assign({}, this.options.formData), options === null || options === void 0 ? void 0 : options.formData);
        this.options = Object.assign(Object.assign(Object.assign({}, this.options), options), { formData: _formData });
    }
    /**
     * Return uploader if exists else throw error
     *
     * @memberOf FileManager
     */
    getUploader() {
        return this._uploader;
    }
    /**
     * Start uploading this file
     *
     *
     * @memberOf FileManager
     */
    upload() {
        let _uploader;
        try {
            _uploader = this.getUploader();
        }
        catch (e) {
            throw new Error('Couldn`t upload because uploader was not defined. ERR_MSG: ' + e.message);
        }
        this._isUploading = true;
        try {
            _uploader.uploadFile(this);
        }
        catch (e) {
            // TODO write error handling
        }
    }
    /**
     * Cancel upload process from this file
     *
     *
     * @memberOf FileManager
     */
    cancel() {
        if (this._isUploading) {
            const uploader = this.getUploader();
            uploader.cancelUploadFile(this);
        }
    }
    _cancel() {
        if (this._isUploading) {
            this._isCancel = true;
            this._isUploaded = false;
            this._isUploading = false;
        }
    }
    /**
     * Remove this FileManger from uploader queue
     *
     *
     * @memberOf FileManager
     */
    remove() {
        let _uploader;
        try {
            _uploader = this.getUploader();
        }
        catch (e) {
            throw new Error('Couldn`t remove because uploader was not defined. ERR_MSG: ' + e.message);
        }
        if (this._isUploading) {
            this.cancel();
        }
        _uploader.removeFile(this);
        this._setFileActive(false);
    }
    isUploaded() {
        return this._isUploaded;
    }
    isUploading() {
        return this._isUploading;
    }
    canUpload() {
        return !this._isUploaded && !this._isUploading && !this._isSuccess && !this._isError;
    }
    handleImageLoad() {
        this._imageLoad = true;
    }
    /**
     * Overwrite functions
     */
    onBeforeUpload() {
        return;
    }
    onProgressSpeed(speed) {
        speed = speed;
    }
    onProgress(progress) {
        progress = progress;
    }
    onSuccess(response, status, headers) {
        response = response;
        status = status;
        headers = headers;
    }
    onError(response, status, headers) {
        response = response;
        status = status;
        headers = headers;
    }
    /**
     *  Internal functions
     */
    _onBeforeUpload() {
        this._isUploading = true;
        this._isUploaded = false;
        this._isSuccess = false;
        this._isCancel = false;
        this._isError = false;
        this._progress$.next(0);
        this.onBeforeUpload();
    }
    _onProgressFileSpeed(speed) {
        this._speed$.next(speed);
        this.onProgressSpeed(speed);
    }
    _onProgress(progress) {
        this._progress$.next(progress);
        this.onProgress(progress);
    }
    _onSuccess(response, status, headers) {
        this._isUploading = false;
        this._isUploaded = true;
        this._isSuccess = true;
        this._isError = false;
        this._response = response;
        this.onSuccess(response, status, headers);
        if (this._uploader.options.removeBySuccess) {
            this.remove();
        }
    }
    _onError(response, status, headers) {
        this._isUploading = false;
        this._isUploaded = false;
        this._isSuccess = false;
        this._isError = true;
        this._response = response;
        this.onError(response, status, headers);
    }
    _setFileActive(check) {
        this._fileActive = check;
    }
}

function instanceOfFileManagerInterface(obj) {
    return [
        'options',
        'progressPercent$',
        'progress$',
        'handleImageLoad',
        'canUpload',
        'isUploading',
        'isUploaded',
        'remove',
        '_cancel',
        'cancel',
        'upload',
        'getUploader',
        'onBeforeUpload',
        'onProgressSpeed',
        'onProgress',
        'onSuccess',
        'onError',
        '_onBeforeUpload',
        '_onProgressFileSpeed',
        '_onProgress',
        '_onSuccess',
        '_onError'
    ].every((key) => key in obj);
}

// tslint:disable:max-line-length
//
// Directive to support dragging and dropping and element onto a div
//
class FileDropDirective {
    //
    // Constructor requires an element reference that instantiated this directive
    //
    constructor(element, renderer) {
        this.element = element;
        this.renderer = renderer;
        this.fileHoverStart = new EventEmitter();
        this.fileHoverEnd = new EventEmitter();
        this.fileAccepted = new EventEmitter();
        this.fileRejected = new EventEmitter();
        this._InputFile = null;
    }
    //
    // Initialisation
    //
    ngOnInit() {
        this._files = [];
        this.renderer.addClass(this.element.nativeElement, 'drop-area');
    }
    ngOnDestroy() {
        if (this._files.length > 0) {
            for (const _file of this._files) {
                if (instanceOfFileManagerInterface(_file)) {
                    if (!_file.isUploaded()) {
                        _file.cancel();
                        this.uploader.removeFile(_file);
                    }
                }
            }
        }
    }
    //
    // Called when the element has content dragged over
    //
    onDragOver(event) {
        // If we're already in the on-drag, don't bother with this
        if (this._InputFile === null) {
            // Get the object being dragged and reference it as a copy action
            this._InputFile = this.getDataTransferObject(event);
            if (this._InputFile === null) {
                return;
            }
            this.renderer.addClass(this.element.nativeElement, 'drop-enter');
            const _dropValueText = 'Drop now';
            this.renderer.setAttribute(this.element.nativeElement, 'dropValueText', _dropValueText);
            // Let the client know
            this.fileHoverStart.emit();
        }
        // Don't propagate
        this.preventAndStopEventPropagation(event);
    }
    //
    // Called when the element has dragged content leave
    //
    onDragLeave(event) {
        // Only bother if we have a file
        if (this._InputFile !== null) {
            this.renderer.removeClass(this.element.nativeElement, 'drop-enter');
            this.renderer.setAttribute(this.element.nativeElement, 'dropValueText', '');
            // Finished with the file
            this._InputFile = null;
            if (event.currentTarget === this.element[0]) {
                return;
            }
            // Let the client know
            this.fileHoverEnd.emit();
        }
        // Don't let it continue
        this.preventAndStopEventPropagation(event);
    }
    //
    // Called when the element has content dropped
    //
    onDrop(event) {
        // Only bother if we have a file
        if (this._InputFile !== null) {
            this.renderer.removeClass(this.element.nativeElement, 'drop-enter');
            this.renderer.setAttribute(this.element.nativeElement, 'dropValueText', '');
            // Let the client know
            this.fileHoverEnd.emit();
            // Update our data
            this._InputFile = this.getDataTransferObject(event);
            if (this._InputFile.files.length === 0) {
                this._InputFile = null;
                return;
            }
            const filesData = this._InputFile.files;
            if (!this.hasFiles(this._InputFile.types)) {
                return;
            }
            this.readFile(filesData);
            this.fileAccepted.emit(this._files);
            // Finished with the file
            this._InputFile = null;
        }
        // Don't let it continue
        this.preventAndStopEventPropagation(event);
    }
    //
    // Stops the drag/drop events propagating
    //
    preventAndStopEventPropagation(event) {
        event.preventDefault();
        event.stopPropagation();
    }
    readFile(_files) {
        let fmObject;
        // tslint:disable-next-line: prefer-for-of
        for (let i = 0; i < _files.length; i++) {
            const file = _files[i];
            try {
                fmObject = new FileManager(file, this.fileOptions, this.uploader);
            }
            catch (e) {
                this.fileRejected.emit(e);
                throw new Error('Something goes wrong e: ' + e.message);
            }
            fmObject.inQueue && this._files.push(fmObject);
        }
    }
    //
    // Returns the file dragged into the directive
    //
    getDataTransferObject(event) {
        return event.dataTransfer ? event.dataTransfer : event.originalEvent.dataTransfer;
    }
    hasFiles(types) {
        if (!types) {
            return false;
        }
        if (types.indexOf) {
            return types.indexOf('Files') !== -1;
        }
        if (types.contains) {
            return types.contains('Files');
        }
        return false;
    }
}
FileDropDirective.decorators = [
    { type: Directive, args: [{
                // Selector required in component HTML
                // tslint:disable-next-line:directive-selector
                selector: '[ngxFile2Drop]'
            },] }
];
FileDropDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: Renderer2 }
];
FileDropDirective.propDecorators = {
    fileHoverStart: [{ type: Output }],
    fileHoverEnd: [{ type: Output }],
    fileAccepted: [{ type: Output }],
    fileRejected: [{ type: Output }],
    fileOptions: [{ type: Input }],
    uploader: [{ type: Input }],
    onDragOver: [{ type: HostListener, args: ['dragover', ['$event'],] }],
    onDragLeave: [{ type: HostListener, args: ['dragleave', ['$event'],] }],
    onDrop: [{ type: HostListener, args: ['drop', ['$event'],] }]
};

// tslint:disable:max-line-length
//
// Directive to support select file from inputfield
//
class FileSelectDirective {
    //
    // Constructor requires an element reference that instantiated this directive
    //
    constructor() {
        this.role = 'input';
        this.type = 'file';
        this.fileHoverStart = new EventEmitter();
        this.fileHoverEnd = new EventEmitter();
        this.fileAccepted = new EventEmitter();
        this.fileRejected = new EventEmitter();
        this._InputFile = null;
    }
    //
    // Initialisation
    //
    ngOnInit() {
        this._files = [];
    }
    ngOnDestroy() {
        if (this._files.length > 0) {
            for (const _file of this._files) {
                if (instanceOfFileManagerInterface(_file)) {
                    if (!_file.isUploaded()) {
                        _file.cancel();
                        this.uploader.removeFile(_file);
                    }
                }
            }
        }
    }
    //
    // Called when the element was choosen
    //
    onChange(event) {
        // Update our data
        this._InputFile = this.getEventTarget(event);
        if (this._InputFile.files.length === 0) {
            this._InputFile = null;
            return;
        }
        const filesData = this._InputFile.files;
        this.readFile(filesData);
        this.fileAccepted.emit(this._files);
        // Finished with the file
        this._InputFile = null;
        // Don't let it continue
        this.preventAndStopEventPropagation(event);
    }
    //
    // Stops the drag/drop events propagating
    //
    preventAndStopEventPropagation(event) {
        event.preventDefault();
        event.stopPropagation();
    }
    readFile(_files) {
        let fmObject;
        // tslint:disable-next-line: prefer-for-of
        for (let i = 0; i < _files.length; i++) {
            const file = _files[i];
            try {
                fmObject = new FileManager(file, this.fileOptions, this.uploader);
            }
            catch (e) {
                if (e.status === 100) {
                    this.fileRejected.emit(e);
                }
                else {
                    this.fileRejected.emit(e);
                }
                throw new Error('Something goes wrong e: ' + e.message);
            }
            fmObject.inQueue && this._files.push(fmObject);
        }
    }
    //
    // Returns the file dragged into the directive
    //
    getEventTarget(event) {
        return event.target;
    }
}
FileSelectDirective.decorators = [
    { type: Directive, args: [{
                // Selector required in component HTML
                // tslint:disable-next-line:directive-selector
                selector: '[ngxFile2Select]'
            },] }
];
FileSelectDirective.ctorParameters = () => [];
FileSelectDirective.propDecorators = {
    role: [{ type: HostBinding, args: ['attr.role',] }],
    type: [{ type: HostBinding, args: ['attr.type',] }],
    fileHoverStart: [{ type: Output }],
    fileHoverEnd: [{ type: Output }],
    fileAccepted: [{ type: Output }],
    fileRejected: [{ type: Output }],
    fileOptions: [{ type: Input }],
    uploader: [{ type: Input }],
    onChange: [{ type: HostListener, args: ['change', ['$event'],] }]
};

const { FileReader: FileReader$1 } = window;
// tslint:disable-next-line:directive-selector
class ImagePreviewDirective {
    constructor(el, renderer2) {
        this.el = el;
    }
    ngOnChanges() {
        const reader = new FileReader$1();
        const el = this.el;
        el.nativeElement.src =
            // tslint:disable-next-line: quotemark
            "data:image/svg+xml;charset=utf-8,%3Csvg xmlns%3D'http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg' viewBox%3D'0 0 200 150'%2F%3E";
        reader.onloadend = () => {
            el.nativeElement.src = reader.result;
        };
        if (this.image) {
            return reader.readAsDataURL(this.image.element);
        }
    }
}
ImagePreviewDirective.decorators = [
    { type: Directive, args: [{ selector: 'img[imgPreview]' },] }
];
ImagePreviewDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: Renderer2 }
];
ImagePreviewDirective.propDecorators = {
    image: [{ type: Input }]
};

/*
 * Convert bytes into largest possible unit.
 * Takes an precision argument that defaults to 2.
 * Usage:
 *   bytes | fileSize:precision
 * Example:
 *   {{ 1024 |  fileSize}}
 *   formats to: 1 KB
*/
class FileSizePipe {
    constructor() {
        this.units = [
            'bytes',
            'KB',
            'MB',
            'GB',
            'TB',
            'PB'
        ];
    }
    transform(bytes = 0, precision = 2) {
        if (!isFinite(bytes)) {
            return '?';
        }
        let unit = 0;
        while (bytes >= 1024) {
            bytes /= 1024;
            unit++;
        }
        return bytes.toFixed(+precision) + ' ' + this.units[unit];
    }
}
FileSizePipe.decorators = [
    { type: Pipe, args: [{ name: 'fileSize' },] }
];

// tslint:disable:directive-selector
class ProgressBarDirective {
    constructor(el, renderer) {
        this.el = el;
        this.renderer = renderer;
    }
    ngOnChanges() {
        const el = this.el;
        el.nativeElement.value = this.progress.percent;
        if (this.progress.speed > 0) {
            const pipe = new FileSizePipe();
            this.renderer.setAttribute(this.el.nativeElement, 'speedText', pipe.transform(this.progress.loaded));
        }
        else {
            this.renderer.setAttribute(this.el.nativeElement, 'speedText', '');
        }
    }
}
ProgressBarDirective.decorators = [
    { type: Directive, args: [{
                selector: 'progress[progressBar]'
            },] }
];
ProgressBarDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: Renderer2 }
];
ProgressBarDirective.propDecorators = {
    progress: [{ type: Input }]
};

// tslint:disable:max-line-length
const { FormData: FormData$1 } = window;
/**
 * Abstract proctol class if someone want to write his own protocol
 */
class Protocol {
    /**
     * Creates an instance of Protocol and for each protocol an own unique ID.
     *
     * @memberOf Protocol
     */
    constructor() {
        this._id = Utils.uniqueID();
        this.progress = new EventEmitter();
        this.load = new EventEmitter();
        this.error = new EventEmitter();
        this.abort = new EventEmitter();
        this._connections = [];
    }
    set connection(obj) {
        const { _file, _connection } = obj;
        if (!this.isConnected(_file)) {
            this._connections.push({
                id: _file.id,
                connection: _connection
            });
        }
    }
    /**
     * Call uploader method _onCompleteFile.
     *
     * @memberOf Protocol
     */
    _onLoad(_file, response, status, headers) {
        const uploader = _file.getUploader();
        const gist = this._isSuccessCode(status);
        const method = (g, f, r, s, h) => {
            if (g) {
                uploader._onSuccessFile(f, r, s, h);
            }
            else {
                uploader._onErrorFile(f, r, s, h);
            }
        };
        method(gist, _file, response, status, headers);
        uploader._onCompleteFile(_file, response, status, headers);
    }
    /**
     * Call uploader methodes _onErrorFile and _onCompleteFile.
     *
     * @memberOf Protocol
     */
    _onError(_file, response, status, headers) {
        const uploader = _file.getUploader();
        uploader._onErrorFile(_file, response, status, headers);
        uploader._onCompleteFile(_file, response, status, headers);
    }
    /**
     * Call uploader methodes _onErrorFile and _onCompleteFile.
     *
     * @memberOf Protocol
     */
    _onAbort(_file, response, status, headers) {
        const uploader = _file.getUploader();
        uploader._onErrorFile(_file, response, status, headers);
        uploader._onCompleteFile(_file, response, status, headers);
    }
    /**
     * Validate response status code.+
     *
     * @memberOf Protocol
     */
    _isSuccessCode(status) {
        return (status >= 200 && status < 300) || status === 304;
    }
    isConnected(_file) {
        for (const _request of this._connections) {
            if (_request.id === _file.id) {
                return _request;
            }
        }
        return false;
    }
    removeConnection(_file) {
        let _request = null;
        for (const _key in this._connections) {
            if (this._connections.hasOwnProperty(_key)) {
                _request = this._connections[_key];
                if (_file.id === _request.id) {
                    this._connections.splice(+_key, 1);
                }
            }
        }
    }
}
/**
 * Standard protocol for server communication (file uploading)
 *
 * @export
 * @extends {Protocol}
 */
class ProtocolXHR extends Protocol {
    constructor() {
        super();
    }
    /**
     * Implementation of the abstract.protocol method `run`
     *
     * @memberOf ProtocolXHR
     */
    run(_file) {
        let _xhr;
        let sendable;
        const uploader = _file.getUploader();
        const _formData = Utils.extendValue(uploader.options.formData, _file.options.formData);
        const _withCredentials = Utils.extendValue(uploader.options.withCredentials, _file.options.withCredentials);
        const _method = Utils.extendValue(uploader.options.method, _file.options.method);
        const _url = Utils.extendValue(uploader.options.url, _file.options.url);
        const _alias = Utils.extendValue(uploader.options.alias, _file.options.alias);
        const _headers = Utils.extendValue(uploader.options.headers, _file.options.headers);
        const _time = new Date().getTime();
        let _load = 0;
        let _speed = 0;
        let _speedToText = null;
        const $filesize = new FileSizePipe();
        _xhr = new XMLHttpRequest();
        this.connection = {
            _file: _file,
            _connection: _xhr
        };
        sendable = new FormData$1();
        if (typeof _formData !== 'undefined') {
            // Only allow Multipart
            for (const key in _formData) {
                if (_formData.hasOwnProperty(key)) {
                    const value = _formData[key];
                    sendable.append(key, value);
                }
            }
        }
        sendable.append(_alias, _file.element, _file.name);
        if (typeof _file.size !== 'number') {
            throw new TypeError('We need the file size.');
        }
        const _progress = (event) => {
            if (event.lengthComputable) {
                const _time2 = new Date().getTime() - _time;
                _load = event.loaded - _load;
                _speed = (_load / _time2) * 1000;
                _speed = parseInt(_speed, 10);
                _speedToText = $filesize.transform(_speed);
                const _obj = {
                    total: event.total,
                    loaded: event.loaded,
                    percent: Math.round((event.loaded / event.total) * 100),
                    speed: _speed,
                    speedToText: _speedToText
                };
                this.progress.emit({ _file: _file, _data: _obj });
            }
        };
        const _loadHandler = (event) => {
            const headers = uploader._parseHeaders(_xhr.getAllResponseHeaders());
            const response = uploader._transformResponse(_xhr.response, headers);
            const status = _xhr.status;
            this.load.emit({ _file: _file, response: response, status: status, headers: headers });
        };
        const _errorHandler = (event) => {
            const headers = uploader._parseHeaders(_xhr.getAllResponseHeaders());
            const response = uploader._transformResponse(_xhr.response, headers);
            const status = _xhr.status;
            this.error.emit({ _file: _file, response: response, status: status, headers: headers });
        };
        const _abortHandler = (event) => {
            const headers = uploader._parseHeaders(_xhr.getAllResponseHeaders());
            const response = uploader._transformResponse(_xhr.response, headers);
            const status = _xhr.status;
            this.abort.emit({ _file: _file, response: response, status: status, headers: headers });
        };
        _xhr.upload.addEventListener('progress', _progress);
        _xhr.onreadystatechange = function (ev) {
            if (this.readyState === 4) {
                if (this.status >= 400) {
                    _errorHandler(ev);
                }
            }
        };
        _xhr.onload = _loadHandler;
        _xhr.addEventListener('abort', _abortHandler);
        _xhr.open(_method, _url, true);
        _xhr.withCredentials = _withCredentials;
        for (const name in _headers) {
            if (_headers.hasOwnProperty(name)) {
                const value = _headers[name];
                _xhr.setRequestHeader(name, value);
            }
        }
        _xhr.send(sendable);
    }
    cancel(_file) {
        const _request = this.isConnected(_file);
        if (!!_request) {
            _request.connection.abort();
            this.removeConnection(_file);
        }
    }
}

var HookTypeEnum;
(function (HookTypeEnum) {
    HookTypeEnum["beforeAddingFile"] = "beforeAddingFile";
    HookTypeEnum["afterAddingFile"] = "afterAddingFile";
    HookTypeEnum["errorAddingFile"] = "errorAddingFile";
    HookTypeEnum["removeFile"] = "removeFile";
    HookTypeEnum["prepareUploadFile"] = "prepareUploadFile";
    HookTypeEnum["progressUploadFile"] = "progressUploadFile";
    HookTypeEnum["progressUploadSpeed"] = "progressUploadSpeed";
    HookTypeEnum["successUploadFile"] = "successUploadFile";
    HookTypeEnum["completeUploadFile"] = "completeUploadFile";
    HookTypeEnum["failedUploadFile"] = "failedUploadFile";
    HookTypeEnum["cancelUploadFile"] = "cancelUploadFile";
    HookTypeEnum["prepareUploadAll"] = "prepareUploadAll";
    HookTypeEnum["progressUploadAll"] = "progressUploadAll";
    HookTypeEnum["completeUploadAll"] = "completeUploadAll";
})(HookTypeEnum || (HookTypeEnum = {}));
class UploaderHook {
    constructor(_hookType, _callback, _priority = 0) {
        this._type = null;
        this._callback = null;
        this._priority = null;
        this._type = _hookType;
        this._callback = _callback;
        this._priority = +_priority;
    }
    get type() {
        return this._type;
    }
    get priority() {
        return this._priority;
    }
    get callback() {
        return this._callback;
    }
}

// tslint:disable: no-unused-expression
/**
 * An abstract class for the transport functionality
 *
 * @export
 */
class Transfer {
    /**
     * Creates an instance of Transfer.
     *
     * @memberOf Transfer
     */
    constructor(type, options) {
        this.type = type;
        this.transferOptionsDefault = {
            url: '',
            alias: 'file',
            headers: {},
            filters: [],
            formData: [],
            autoUpload: false,
            method: 'POST',
            removeBySuccess: false,
            enableCors: false,
            withCredentials: false,
            uniqueFiles: false
        };
        this._$queue = new BehaviorSubject([]);
        this._queue = [];
        const div = document.createElement('div');
        this._isHTML5 = !!(File && FormData && FileReader);
        this._isDragAndDrop = 'draggable' in div || ('ondragstart' in div && 'ondrop' in div) ? true : false;
        this._id = Utils.uniqueID();
        this._hooks = [];
        if (!this._isHTML5) {
            throw new Error(`Your browser doesn't support HTML5. Our NgxFileUploader can't work here.`);
        }
        this.options = Object.assign({}, this.transferOptionsDefault, options);
    }
    get id() {
        return this._id;
    }
    get queue$() {
        return this._$queue.asObservable();
    }
    get queueObs() {
        return this._queue;
    }
    /**
     * Bind options to FileManagerInterface
     *
     * @memberOf Transfer
     */
    bindOptions(options) {
        this.options = Object.assign(Object.assign({}, this.options), options);
    }
    /**
     * It gives HTML5 check back
     *
     * @memberOf Transfer
     */
    isHTML5() {
        return this._isHTML5;
    }
    /**
     * It gives DragAndDrop check back
     *
     * @memberOf Transfer
     */
    isDragAndDrop() {
        return this._isDragAndDrop;
    }
    /**
     * To implement a hock
     *
     * @memberOf Transfer
     */
    hook(hook) {
        if (this._hookExists(hook) === -1) {
            this._hooks.push(hook);
            this._hooks.sort((a, b) => {
                if (a.type !== b.type) {
                    if (a.type < b.type) {
                        return -1;
                    }
                    if (a.type > b.type) {
                        return 1;
                    }
                }
                else {
                    if (a.priority > b.priority) {
                        return -1;
                    }
                    if (a.priority < b.priority) {
                        return 1;
                    }
                    return 0;
                }
            });
        }
        else {
            throw new Error(`Hook with the Type [${hook.type}] and the same callback still exists.`);
        }
    }
    /**
     *  Remove a hook
     *
     * @memberOf Transfer
     */
    removeHook(hook) {
        const key = this._hookExists(hook);
        if (key >= 0) {
            this._hooks.splice(key, 1);
            return true;
        }
        return false;
    }
    activeHooks() {
        return this._hooks.map((_hook) => `${HookTypeEnum[_hook.type]}->prio:${_hook.priority}`);
    }
    addFile(file) {
        let _valid = false;
        try {
            _valid = this.validate(file);
        }
        catch (e) {
            _valid = false;
        }
        if (!_valid) {
            this._onAddFileError(file);
            return false;
        }
        const queue = this._queue;
        if (this.options.uniqueFiles) {
            if (this.inQueue(file) === -1) {
                this._runHook(HookTypeEnum.beforeAddingFile, file);
                queue.push(file);
                this._$queue.next(queue);
                this._onAddFile(file);
                this._runHook(HookTypeEnum.afterAddingFile, file);
                this.options.autoUpload && file.upload();
                return true;
            }
        }
        else {
            this._runHook(HookTypeEnum.beforeAddingFile, file);
            queue.push(file);
            this._$queue.next(queue);
            this._onAddFile(file);
            this._runHook(HookTypeEnum.afterAddingFile, file);
            this.options.autoUpload && file.upload();
            return true;
        }
        this._onAddFileError(file);
        return false;
    }
    removeFile(_file) {
        const key = this.inQueue(_file);
        if (key !== -1) {
            this._queue.splice(key, 1);
            this._$queue.next(this._queue);
            this._onRemoveFile(_file);
            return true;
        }
        return false;
    }
    inQueue(_file) {
        return this._queue.findIndex((obj) => {
            return obj.id === _file.id || obj.name + obj.type + obj.size === _file.name + _file.type + _file.size;
        });
    }
    addFilter(_filter) {
        if (!this._filterExists(_filter)) {
            this.options.filters.push(_filter);
        }
    }
    validate(_file) {
        if (this.options.filters.length > 0) {
            return this.options.filters.some((_filter) => _filter.validate(_file));
        }
        return true;
    }
    setProtocol(_protocol) {
        this._protocol = _protocol;
    }
    getProtocol() {
        return this._protocol;
    }
    /**
     * Validate response status code.
     *
     * @memberOf Transfer
     */
    isSuccessCode(status) {
        return (status >= 200 && status < 300) || status === 304;
    }
    /**
     * Upload functions
     */
    upload() {
        this._onBeforeUploadAll();
        for (const _file of this.queueObs) {
            this.uploadFile(_file);
        }
    }
    cancel() {
        for (const _file of this.queueObs) {
            this.cancelUploadFile(_file);
        }
    }
    remove() {
        const _queue = [...this.queueObs];
        for (const _file of _queue) {
            this.removeFile(_file);
        }
    }
    uploadFile(item) {
        if (this.inQueue(item) === -1) {
            this.addFile(item);
        }
        this._onBeforeUpload(item);
        this.getProtocol().run(item);
    }
    cancelUploadFile(item) {
        if (this.inQueue(item) !== -1) {
            item.isUploading && this.getProtocol().cancel(item);
            item.isUploading && item._cancel();
        }
    }
    /**
     * Overwrite functions
     */
    onAddFile(_file) {
        return;
    }
    onAddFileError(_file) {
        return;
    }
    onRemoveFile(_file) {
        return;
    }
    onBeforeUploadAll(_uploader) {
        return;
    }
    onBeforeUpload(_file) {
        return;
    }
    onProgress(_uploader, _progress) {
        return;
    }
    onProgressFile(_file, _progress) {
        return;
    }
    onProgressFileSpeed(_file, _progress) {
        return;
    }
    onSuccess(_file, _response, _status, _headers) {
        return;
    }
    onError(_file, _response, _status, _headers) {
        return;
    }
    onCancel(_file, _response, _status, _headers) {
        return;
    }
    onComplete(_file, _response, _status, _headers) {
        return;
    }
    onCompleteAll(_uploader) {
        return;
    }
    /**
     * Internal functions
     */
    _onAddFile(_file) {
        this.onAddFile(_file);
    }
    _onAddFileError(_file) {
        this._runHook(HookTypeEnum.errorAddingFile, _file);
        this.onAddFileError(_file);
    }
    _onRemoveFile(_file) {
        this._runHook(HookTypeEnum.removeFile, _file);
        this.onRemoveFile(_file);
    }
    _onBeforeUploadAll() {
        this._runHook(HookTypeEnum.prepareUploadAll, this);
        this.onBeforeUploadAll(this);
    }
    _onBeforeUpload(_file) {
        this._runHook(HookTypeEnum.prepareUploadFile, _file);
        _file._onBeforeUpload();
        this.onBeforeUpload(_file);
    }
    _onProgressFileSpeed(_file, _speed) {
        this._runHook(HookTypeEnum.progressUploadSpeed, _file, _speed);
        _file._onProgressFileSpeed(_speed);
        this.onProgressFileSpeed(_file, _speed);
    }
    _onProgressFile(_file, _progress) {
        this._runHook(HookTypeEnum.progressUploadFile, _file, _progress);
        _file._onProgress(_progress);
        this.onProgressFile(_file, _progress);
        this._onProgress();
    }
    _onProgress() {
        const queue = this._queue;
        if (queue.length > 0) {
            let percent = 0;
            for (const file of queue) {
                if (file.success || file.error) {
                    percent += 100;
                }
                else if (file.isUploading) {
                    percent += file.progress;
                }
            }
            percent = Math.floor(percent / queue.length);
            this._runHook(HookTypeEnum.progressUploadAll, percent);
            this.onProgress(this, percent);
            percent >= 100 && this._onCompleteAll();
        }
    }
    _onSuccessFile(_file, response, status, headers) {
        this._runHook(HookTypeEnum.successUploadFile, _file, response, status, headers);
        _file._onSuccess(response, status, headers);
        this.onSuccess(_file, response, status, headers);
    }
    _onErrorFile(_file, response, status, headers) {
        this._runHook(HookTypeEnum.failedUploadFile, _file, response, status, headers);
        _file._onError(response, status, headers);
        this.onError(_file, response, status, headers);
    }
    _onCancelFile(_file, response, status, headers) {
        this._runHook(HookTypeEnum.cancelUploadFile, _file, response, status, headers);
        _file._onError(response, status, headers);
        this.onCancel(_file, response, status, headers);
    }
    _onCompleteFile(_file, response, status, headers) {
        this._runHook(HookTypeEnum.completeUploadFile, _file, response, status, headers);
        this._onProgress();
        this.onComplete(_file, response, status, headers);
    }
    _onCompleteAll() {
        this._runHook(HookTypeEnum.completeUploadAll, this);
        this.onCompleteAll(this);
    }
    _parseHeaders(headers) {
        const parsed = {};
        let key, val, i;
        if (!headers) {
            return parsed;
        }
        const incomeHeaders = headers.split('\n');
        for (const line of incomeHeaders) {
            i = line.indexOf(':');
            key = line
                .slice(0, i)
                .trim()
                .toLowerCase();
            val = line.slice(i + 1).trim();
            if (key) {
                parsed[key] = parsed[key] ? parsed[key] + ', ' + val : val;
            }
        }
        return parsed;
    }
    _transformResponse(response, headers) {
        headers = {};
        return response;
    }
    _runHook(type, ...args) {
        for (const key in this._hooks) {
            if (this._hooks.hasOwnProperty(key)) {
                const obj = this._hooks[key];
                if (obj.type === type) {
                    switch (type) {
                        case HookTypeEnum.beforeAddingFile:
                        case HookTypeEnum.prepareUploadAll:
                        case HookTypeEnum.prepareUploadFile:
                        case HookTypeEnum.afterAddingFile:
                        case HookTypeEnum.errorAddingFile:
                        case HookTypeEnum.completeUploadAll:
                        case HookTypeEnum.progressUploadAll:
                        case HookTypeEnum.removeFile:
                            {
                                obj.callback(args[0]);
                            }
                            break;
                        case HookTypeEnum.progressUploadFile:
                        case HookTypeEnum.progressUploadSpeed:
                            {
                                obj.callback(args[0], args[1]);
                            }
                            break;
                        case HookTypeEnum.cancelUploadFile:
                        case HookTypeEnum.successUploadFile:
                        case HookTypeEnum.failedUploadFile:
                        case HookTypeEnum.completeUploadFile:
                            {
                                obj.callback(args[0], args[1], args[2], args[3]);
                            }
                            break;
                        default: {
                            throw new Error(`hookType unknown or not defined`);
                        }
                    }
                }
            }
        }
    }
    _hookExists(hook) {
        for (const key in this._hooks) {
            if (this._hooks.hasOwnProperty(key)) {
                const obj = this._hooks[key];
                if (obj.type === hook.type && '' + obj.callback === '' + hook.callback) {
                    return +key;
                }
            }
        }
        return -1;
    }
    _filterExists(filter) {
        const _filters = this.options.filters;
        for (const _filter of _filters) {
            if (_filter.name === filter.name) {
                return true;
            }
        }
        return false;
    }
}

class FileUploader extends Transfer {
    constructor(_options, _protocol) {
        super('FileUploader', _options);
        this._subs = [];
        if (typeof _protocol === 'undefined') {
            this.setProtocol(new ProtocolXHR());
        }
        else {
            this.setProtocol(_protocol);
        }
        this.init();
    }
    init() {
        this.destroy();
        const sub1 = this.getProtocol().progress.subscribe((obj) => {
            const { _file, _data } = obj;
            if (instanceOfFileManagerInterface(_file)) {
                this._onProgressFile(_file, _data.percent);
                this._onProgressFileSpeed(_file, _data);
            }
        }, (error) => {
            throw new Error(error);
        });
        this._subs.push(sub1);
        const sub2 = this.getProtocol().load.subscribe((obj) => {
            const { _file, response, status, headers } = obj;
            if (instanceOfFileManagerInterface(_file)) {
                const uploader = _file.getUploader();
                const gist = this.isSuccessCode(status);
                const method = (g, f, r, s, h) => {
                    if (g) {
                        uploader._onSuccessFile(f, r, s, h);
                    }
                    else {
                        uploader._onErrorFile(f, r, s, h);
                    }
                };
                method(gist, _file, response, status, headers);
                uploader._onCompleteFile(_file, response, status, headers);
            }
        }, (error) => {
            throw new Error(error);
        });
        this._subs.push(sub2);
        const sub3 = this.getProtocol().error.subscribe((obj) => {
            const { _file, response, status, headers } = obj;
            if (instanceOfFileManagerInterface(_file)) {
                const uploader = _file.getUploader();
                uploader._onErrorFile(_file, response, status, headers);
                uploader._onCompleteFile(_file, response, status, headers);
            }
        }, (error) => {
            throw new Error(error);
        });
        this._subs.push(sub3);
        const sub4 = this.getProtocol().abort.subscribe((obj) => {
            const { _file, response, status, headers } = obj;
            if (instanceOfFileManagerInterface(_file)) {
                const uploader = _file.getUploader();
                uploader._onCancelFile(_file, response, status, headers);
                uploader._onCompleteFile(_file, response, status, headers);
            }
        }, (error) => {
            throw new Error(error);
        });
        this._subs.push(sub4);
    }
    destroy() {
        for (const _sub of this._subs) {
            _sub.unsubscribe();
        }
        this._subs = [];
    }
}

var filterType;
(function (filterType) {
    filterType["regex"] = "regex";
    filterType["callback"] = "callback";
})(filterType || (filterType = {}));
class FileFilter {
    constructor(name, _data, _regCheck = 'name') {
        this._name = '';
        this._regex = null;
        this._regCheck = null;
        this._callback = null;
        this._type = null;
        this._name = name;
        if (_data instanceof RegExp) {
            this._type = filterType.regex;
            this._regex = _data;
            this._regCheck = _regCheck;
            return;
        }
        else if (_data instanceof Function) {
            this._type = filterType.callback;
            this._callback = _data;
            return;
        }
        throw new Error('FilterData is not defined.');
    }
    get name() {
        return this._name;
    }
    get type() {
        return this._type;
    }
    validate(file) {
        let _valid = false;
        if (this._type === filterType.regex) {
            return this._regexpCheck(file);
        }
        else if (this._type === filterType.callback) {
            _valid = this._callback(file);
        }
        else {
            throw new Error('Filter type is not defined.');
        }
        return _valid;
    }
    _regexpCheck(file) {
        let _valid = false;
        switch (this._regCheck) {
            case 'name':
                _valid = file.object.name.match(this._regex) != null;
                break;
            case 'type':
                _valid = file.object.type.match(this._regex) != null;
                break;
            case 'date':
                // Format for validation is `2017-01-01 10:10:10`
                const _checkDate = new DatePipe('en-US').transform(file.object.lastModifiedDate, 'yyyy-MM-dd hh:mm:ss');
                if (_checkDate && _checkDate.match(this._regex)) {
                    _valid = true;
                }
                break;
            default: {
                throw new Error('RegExp can only check on `name | type | date`.');
            }
        }
        return _valid;
    }
}

class AngularFileUploadProtocol extends Protocol {
    constructor(_httpClient) {
        super();
        this._httpClient = _httpClient;
        this.ngUnsubscribe = new Subject();
    }
    /**
     * Implementation of the abstract.protocol method `run`
     *
     * @memberOf ProtocolXHR
     */
    run(_file) {
        return __awaiter(this, void 0, void 0, function* () {
            let sendable;
            const uploader = _file.getUploader();
            const _formData = Utils.extendValue(uploader.options.formData, _file.options.formData);
            const _withCredentials = Utils.extendValue(uploader.options.withCredentials, _file.options.withCredentials);
            const _method = Utils.extendValue(uploader.options.method, _file.options.method);
            const _url = Utils.extendValue(uploader.options.url, _file.options.url);
            const _alias = Utils.extendValue(uploader.options.alias, _file.options.alias);
            const _headers = Utils.extendValue(uploader.options.headers, _file.options.headers, { 'Content-Type': 'multipart/form-data' });
            const time = new Date().getTime();
            this.connection = {
                _file: _file,
                _connection: null
            };
            sendable = new FormData();
            if (typeof _formData !== 'undefined') {
                // Only allow Multipart
                for (const obj of _formData) {
                    for (const key in obj) {
                        if (obj.hasOwnProperty(key)) {
                            const value = obj[key];
                            sendable.append(key, value);
                        }
                    }
                }
            }
            sendable.append(_alias, _file.element, _file.name);
            if (typeof _file.size !== 'number') {
                throw new TypeError('We need the file size.');
            }
            const _req = new HttpRequest(_method, _url, sendable, {
                headers: _headers,
                reportProgress: true,
                withCredentials: _withCredentials
            });
            const _post = this._httpClient.request(_req);
            yield this.handleMethode(_file, _post, time);
        });
    }
    handleMethode(_file, requestHandle, time) {
        return __awaiter(this, void 0, void 0, function* () {
            let _load = 0;
            let _speed = 0;
            let _speedToText = null;
            const _$filesize = new FileSizePipe();
            let _headers, _response, _status, _request;
            return requestHandle
                .pipe(takeUntil(this.ngUnsubscribe), map((event) => {
                switch (event.type) {
                    case HttpEventType.UploadProgress:
                        const _time = new Date().getTime() - time;
                        _load = event.loaded - _load;
                        _speed = (_load / _time) * 1000;
                        _speed = parseInt(_speed, 10);
                        _speedToText = _$filesize.transform(_speed);
                        const _obj = {
                            total: event.total,
                            loaded: event.loaded,
                            percent: Math.round((event.loaded / event.total) * 100),
                            speed: _speed,
                            speedToText: _speedToText
                        };
                        _request = this.isConnected(_file);
                        if (!!_request) {
                            this.progress.emit({ _file: _file, _data: _obj });
                        }
                        return event;
                    case HttpEventType.Response:
                        _headers = event.headers;
                        _response = event.body;
                        _status = event.status;
                        _request = this.isConnected(_file);
                        if (!!_request) {
                            this.load.emit({ _file: _file, response: _response, status: _status, headers: _headers });
                        }
                        return event;
                    default:
                        return event;
                }
            }), catchError((operation = 'operation', result) => {
                return (error) => {
                    console.log(`${operation} failed: ${error.message}`);
                    this.error.emit({ _file: _file, response: result, status: error.status, headers: {} });
                    console.error(error);
                    return of(result);
                };
            }))
                .toPromise();
        });
    }
    cancel(_file) {
        const _request = this.isConnected(_file);
        if (!!_request) {
            this.ngUnsubscribe.next();
            this.ngUnsubscribe.complete();
            this.abort.emit({ _file: _file, response: {}, status: 999, headers: {} });
            this.removeConnection(_file);
        }
    }
}

const FILE_UPLOADER_CONFIG = new InjectionToken(null);
class FileUploadService {
    constructor(_httpClient, _config) {
        this._httpClient = _httpClient;
        this._config = _config;
        this._uploadServices = {};
        if (_config && _config.url) {
            this.registerUploadService('default', new FileUploader(_config));
            this.useAngularProtocol('default');
        }
    }
    registerUploadService(uploaderKey, uploader) {
        if (this._uploadServices[uploaderKey] != null) {
            throw new Error(`Upload service with the key [${uploaderKey}] exists.`);
        }
        if (this._config) {
            uploader.bindOptions(Object.assign(Object.assign({}, this._config), uploader.options));
        }
        this._uploadServices[uploaderKey] = {
            uploader
        };
    }
    setHook(uploaderKey, type, fnCallback, priority = 0) {
        this._checkUploader(uploaderKey);
        this._uploadServices[uploaderKey].uploader.hook(new UploaderHook(type, fnCallback, priority));
    }
    useAngularProtocol(uploaderKey) {
        this._checkUploader(uploaderKey);
        this._uploadServices[uploaderKey].uploader.setProtocol(new AngularFileUploadProtocol(this._httpClient));
        this._uploadServices[uploaderKey].uploader.init();
    }
    useDefaultProtocol(uploaderKey) {
        this._checkUploader(uploaderKey);
        this._uploadServices[uploaderKey].uploader.setProtocol(new ProtocolXHR());
        this._uploadServices[uploaderKey].uploader.init();
    }
    getUploader(uploaderKey) {
        this._checkUploader(uploaderKey);
        return this._uploadServices[uploaderKey].uploader;
    }
    addFile(uploaderKey, file) {
        this._checkUploader(uploaderKey);
        file.bindUploader(this._uploadServices[uploaderKey].uploader);
    }
    addFiles(uploaderKey, files) {
        this._checkUploader(uploaderKey);
        for (const _file of files) {
            _file.bindUploader(this._uploadServices[uploaderKey].uploader);
        }
    }
    startAll(uploaderKey) {
        this._checkUploader(uploaderKey);
        this._uploadServices[uploaderKey].uploader.upload();
    }
    startFile(uploaderKey, file) {
        this._checkUploader(uploaderKey);
        if (file.getUploader() == null) {
            file.bindUploader(this._uploadServices[uploaderKey].uploader);
        }
        this._uploadServices[uploaderKey].uploader.uploadFile(file);
    }
    cancelAll(uploaderKey) {
        this._checkUploader(uploaderKey);
        this._uploadServices[uploaderKey].uploader.cancel();
    }
    cancelFile(uploaderKey, file) {
        this._checkUploader(uploaderKey);
        this._uploadServices[uploaderKey].uploader.cancelUploadFile(file);
    }
    /** Internal Function */
    _checkUploader(uploaderKey) {
        if (this._uploadServices[uploaderKey] == null) {
            throw new Error(`Upload service with the key [${uploaderKey}] doesn't exists.`);
        }
    }
}
FileUploadService.decorators = [
    { type: Injectable }
];
FileUploadService.ctorParameters = () => [
    { type: HttpClient },
    { type: undefined, decorators: [{ type: Inject, args: [FILE_UPLOADER_CONFIG,] }, { type: Optional }] }
];

const providers = [FileUploadService];
const directives = [FileDropDirective, FileSelectDirective, ImagePreviewDirective, ProgressBarDirective];
const pipes = [FileSizePipe];
class NgxFileUploaderModule {
    static forRoot(parameters = {}) {
        return {
            ngModule: NgxFileUploaderModule,
            providers: [...providers, { provide: FILE_UPLOADER_CONFIG, useValue: parameters }]
        };
    }
}
NgxFileUploaderModule.decorators = [
    { type: NgModule, args: [{
                declarations: [...directives, ...pipes],
                imports: [],
                exports: [...directives, ...pipes]
            },] }
];

/*
 * Public API Surface of ngx-file-uploader
 */

/**
 * Generated bundle index. Do not edit.
 */

export { AngularFileUploadProtocol, FILE_UPLOADER_CONFIG, FileDropDirective, FileFilter, FileManager, FileSelectDirective, FileSizePipe, FileUploadService, FileUploader, HookTypeEnum, ImagePreviewDirective, NgxFileUploaderModule, ProgressBarDirective, Protocol, ProtocolXHR, Transfer, UploaderHook, Utils, filterType, instanceOfFileManagerInterface, FileDropDirective as ɵa, FileSelectDirective as ɵb, ImagePreviewDirective as ɵc, ProgressBarDirective as ɵd, FileSizePipe as ɵe, FILE_UPLOADER_CONFIG as ɵf, FileUploadService as ɵg };
//# sourceMappingURL=uniprank-ngx-file-uploader.js.map
